﻿using Newtonsoft.Json;
using Newtonsoft.Json.Linq;
using Newtonsoft.Json.Schema;
using System.Collections.Generic;
using System.IO;

namespace GenLib.Data
{
    /// <summary>
    /// This class provides to load input data from JSON file and validate them.
    /// </summary>
    public class DataLoader
    {
        /// <summary>
        /// Root of the input data.
        /// </summary>
        public DataObject Root { get; set; } = null;

        /// <summary>
        /// Input JSON file path.
        /// </summary>
        private string Path { get; set; }

        /// <summary>
        /// Constructor of the DataLoader class.
        /// </summary>
        /// <param name="path">Input JSON file path.</param>
        public DataLoader(string path)
        {
            Path = path;
        }

        /// <summary>
        /// This method loads input data from JSON file.
        /// </summary>
        public void LoadDataJson()
        {
            if (!ValidateDataJSON(Path))
            {
                throw new DataException(ErrorType.DATA_FILE_INVALID_STRUCTURE);
            }

            using (StreamReader r = new StreamReader(Path))
            {
                string json = r.ReadToEnd();
                Root = JsonConvert.DeserializeObject<DataObject>(json);
            }

        }

        /// <summary>
        /// This method validates input data JSON file according to defined JSON schema.
        /// </summary>
        /// <param name="path">Input JSON file path.</param>
        /// <returns>true - if input JSON file is valid, false - otherwise.</returns>
        private bool ValidateDataJSON(string path)
        {
            string data = File.ReadAllText(path);
            string schema = Properties.Resources.data_schema.ToString();
            JObject model;
            JSchema json_schema;

            try
            {
                model = JObject.Parse(data);
                json_schema = JSchema.Parse(schema);
                IList<string> messages;
                return model.IsValid(json_schema, out messages);
            }
            catch (JsonReaderException e)
            {
                throw new DataException(ErrorType.DATA_FILE_INVALID_STRUCTURE + " " + e.Message);
            }
        }

    }
}
