﻿using System.IO;
using System.Text.RegularExpressions;
using Word = Microsoft.Office.Interop.Word;

namespace GenLib.Document
{
    /// <summary>
    /// This class provides to preprocess Word document.
    /// </summary>
    public class DocumentPreprocessor
    {
        /// <summary>
        /// Regular expression for recognize include directive in Word document.
        /// </summary>
        private static readonly Regex INCLUDE_REGEX = new Regex(@"#include[\s]*\(""(?<path>[^\)]+)""\)");

        /// <summary>
        /// Instance of current Word document.
        /// </summary>
        public Word.Document Document { get; set; }

        /// <summary>
        /// Constructor of the DocumentPreprocessor class.
        /// </summary>
        /// <param name="document">Instance of current Word document.</param>
        public DocumentPreprocessor(Word.Document document)
        {
            Document = document;
        }

        /// <summary>
        /// This method starts preprocessing of Word document.
        /// </summary>
        public void Preprocess()
        {
            ProcessIncludeDirectives(Document);
        }

        /// <summary>
        /// This method processing include directives in Word document.
        /// </summary>
        /// <param name="document">Instance of current Word document.</param>
        private void ProcessIncludeDirectives(Word.Document document)
        {
            MatchCollection matches = INCLUDE_REGEX.Matches(document.Content.Text);

            while (matches != null && matches.Count > 0)
            {
                for (int i = matches.Count - 1; i >= 0; i--)
                {
                    Match currentMatch = matches[i];

                    GroupCollection groups = currentMatch.Groups;

                    ReplaceIncludeDirectives(document, groups["path"].Value, currentMatch.Index, currentMatch.Index + currentMatch.Length);
                }

                matches = INCLUDE_REGEX.Matches(document.Content.Text);
            }
        }

        /// <summary>
        /// This method replaces include directives in Word document by content 
        /// of subtemplate defined in include directive.
        /// </summary>
        /// <param name="mainDocument">Current instance of Word document.</param>
        /// <param name="path">Path of subtemplate.</param>
        /// <param name="startIndex">Start index of include directive in Word document.</param>
        /// <param name="endIndex">End index of include directive in Word document</param>
        private void ReplaceIncludeDirectives(Word.Document mainDocument, string path, int startIndex, int endIndex)
        {
            if (!Path.IsPathRooted(path))
            {
                path = Path.GetFullPath(path);
            }

            Word.Document documentToInclude = DocumentUtil.LoadWordFile(path, true);

            Word.Range documentToIncludeContent = documentToInclude.Range(documentToInclude.Content.Start, documentToInclude.Content.End - 1);
            documentToIncludeContent.Copy();

            Word.Range toPaste = mainDocument.Range(startIndex, endIndex);
            toPaste.Paste();

            documentToInclude.Close();
        }
    }
}
