﻿using Antlr4.Runtime;
using GenLib.Data;
using GenLib.Entity;
using GenLib.Visitor;

namespace GenLib
{
    /// <summary>
    /// This class provides to generate MS Word documents and analyze document templates. 
    /// The generation is conditioned by input data and document template.
    /// This class is according to design pattern Singleton.
    /// </summary>
    public class Generator
    {
        /// <summary>
        /// Constructor of the Generator class.
        /// </summary>
        private Generator() { }

        /// <summary>
        /// This attribute contains instance of the Generator class.
        /// </summary>
        private static readonly Generator INSTANCE = new Generator();

        /// <summary>
        /// Property of the INSTANCE attribute.
        /// </summary>
        public static Generator Instance
        {
            get { return INSTANCE; }
        }

        /// <summary>
        /// This method provides to generate final MS Word document 
        /// from document template (inputText) and input data (dataRoot).
        /// </summary>
        /// <param name="inputText">Text of the document template.</param>
        /// <param name="dataRoot">Root of the input data tree.</param>
        /// <returns>Text of the generated document.</returns>
        public string Generate(string inputText, DataObject dataRoot)
        {
            AntlrInputStream inputStream = new AntlrInputStream(inputText.ToString());
            DocsGenLexer docsGenLexer = new DocsGenLexer(inputStream);
            CommonTokenStream commonTokenStream = new CommonTokenStream(docsGenLexer);
            DocsGenParser docsGenParser = new DocsGenParser(commonTokenStream);

            docsGenParser.RemoveErrorListeners();
            docsGenParser.AddErrorListener(SyntaxErrorListener.Instance);

            DocsGenParser.ParseContext parseContext = docsGenParser.parse();
            Parse parse = new ParseVisitor().Visit(parseContext);

            return new DocumentBuilder(parse, dataRoot).BuildText();
        }

        /// <summary>
        /// This method provides to analyze document template in MS Word format.
        /// </summary>
        /// <param name="inputText">Text of the document template.</param>
        /// <returns>Root of the parse tree.</returns>
        public Parse Analyze(string inputText)
        {
            AntlrInputStream inputStream = new AntlrInputStream(inputText.ToString());
            DocsGenLexer docsGenLexer = new DocsGenLexer(inputStream);
            CommonTokenStream commonTokenStream = new CommonTokenStream(docsGenLexer);
            DocsGenParser docsGenParser = new DocsGenParser(commonTokenStream);

            docsGenParser.RemoveErrorListeners();
            docsGenParser.AddErrorListener(SyntaxErrorListener.Instance);

            DocsGenParser.ParseContext parseContext = docsGenParser.parse();
            Parse parse = new ParseVisitor().Visit(parseContext);

            return parse;
        }
    }
}
