﻿using System.Collections.Generic;
using System.Drawing;
using System.Windows.Forms;
using TemplateBuilder.Model.Utils;

namespace TemplateBuilder.Forms
{
    /// <summary>
    /// This partial class serves actions for ErrorTaskPane. 
    /// The form mediates GUI for the view of syntax errors.
    /// </summary>
    public partial class ErrorTaskPane : UserControl
    {
        /// <summary>
        /// Image with 'OK' icon.
        /// </summary>
        private Image OkImage { get; set; }

        /// <summary>
        /// Image with 'Error' icon.
        /// </summary>
        private Image ErrorImage { get; set; }

        /// <summary>
        /// Set of the syntax errors.
        /// </summary>
        private HashSet<string> Errors { get; set; }

        /// <summary>
        /// Constructor of the ErrorTaskPane class.
        /// </summary>
        public ErrorTaskPane()
        {
            InitializeComponent();
            InitializeCustomComponent();
        }

        /// <summary>
        /// This method initializes custom components of the ErrorTaskPane.
        /// </summary>
        public void InitializeCustomComponent()
        {
            OkImage = ImageConvertor.Convert(Globals.ThisAddIn.Application.CommandBars.GetImageMso("MenuAcceptInvitation", 16, 16));
            ErrorImage = ImageConvertor.Convert(Globals.ThisAddIn.Application.CommandBars.GetImageMso("MenuDeclineInvitation", 16, 16));
            tableLayoutPanel1.CellBorderStyle = TableLayoutPanelCellBorderStyle.None;
            Errors = new HashSet<string>();
            RemoveErrors();
        }

        /// <summary>
        /// This method adds syntax error to the ErrorTaskPane with error message.
        /// </summary>
        /// <param name="errorMessage">Error message.</param>
        public void AddError(string errorMessage)
        {
            if (ContainsError(errorMessage))
            {
                return;
            }

            statusImage.Image = ErrorImage;
            statusLbl.Text = "Errors";

            int lastRowIndex = tableLayoutPanel1.RowCount;

            tableLayoutPanel1.RowCount++;

            tableLayoutPanel1.RowStyles.Insert(lastRowIndex, new RowStyle(SizeType.Absolute, 20));

            tableLayoutPanel1.Controls.Add(new Label()
            {
                Size = new Size(929, 30),
                TextAlign = ContentAlignment.MiddleLeft,
                Text = errorMessage
            }, 0, lastRowIndex);

            Errors.Add(errorMessage);

        }

        /// <summary>
        /// This method resolves if ErrorTaskPane already contains the error message.
        /// </summary>
        /// <param name="errorMessage">Error message.</param>
        /// <returns>true - if ErrorTaskPane already contains the error message; false - otherwise.</returns>
        private bool ContainsError(string errorMessage)
        {
            if (Errors == null || Errors.Count <= 0) 
            {
                return false;
            }

            if (Errors.Contains(errorMessage))
            {
                return true;
            }

            return false;
        }

        /// <summary>
        /// This method removes all syntax errors from the ErrorTaskPane.
        /// </summary>
        public void RemoveErrors()
        {
            statusImage.Image = OkImage;
            statusLbl.Text = "No Errors";

            for (int i = 0; i < tableLayoutPanel1.RowCount; i++)
            {
                Control currentControl = tableLayoutPanel1.GetControlFromPosition(0, i);
                tableLayoutPanel1.Controls.Remove(currentControl);
            }

            tableLayoutPanel1.RowCount = 0;

            if (Errors != null)
            {
                Errors.Clear();
            }
        }
    }
}
