﻿using GenLib.Document;
using System;
using System.ComponentModel;
using System.IO;
using System.Threading;
using System.Windows.Forms;
using Word = Microsoft.Office.Interop.Word;

namespace TemplateBuilder.Forms
{
    /// <summary>
    /// This partial class serves actions for GenerateForm. 
    /// The form mediates GUI for the generation a new Word document.
    /// </summary>
    public partial class GenerateForm : Form
    {
        /// <summary>
        /// Instance of global WinForms dialogs.
        /// </summary>
        private Dialogs _dialogs = null;

        /// <summary>
        /// Instance of opened Word document template.
        /// </summary>
        private Word.Document _currentTemplate = null;

        /// <summary>
        /// Input data file path.
        /// </summary>
        private string _dataFilePath = null;

        /// <summary>
        /// New generated Word document file path.
        /// </summary>
        private string _generatedDocFilePath = null;

        /// <summary>
        /// Flag: true - if input data file path is chosen in dialog; false - otherwise.
        /// </summary>
        private bool _isDataFilePathChosen = false;

        /// <summary>
        /// Flag: true - if new generated Word document file path is chosen in dialog; false - otherwise.
        /// </summary>
        private bool _isGeneratedDocFilePathChosen = false;

        /// <summary>
        /// Instance of BacgroundWorker for generation new Word document.
        /// </summary>
        private BackgroundWorker _generateDocumentBW = null;

        /// <summary>
        /// Instance of WaitingForm for view of generating new document progress.
        /// </summary>
        private WaitingForm _waitingForm = null;

        /// <summary>
        /// Flag: true - if pathes of input data file and generated document file are chosen; false - otherwise.
        /// </summary>
        private bool IsPathChosen
        {
            get
            {
                if (_isDataFilePathChosen && _isGeneratedDocFilePathChosen)
                {
                    return true;
                }

                return false;
            }
        }

        /// <summary>
        /// Constructor of the GenerateForm class.
        /// </summary>
        public GenerateForm()
        {
            _dialogs = Globals.ThisAddIn.Dialogs;
            _currentTemplate = Globals.ThisAddIn.Application.ActiveDocument;
            _generateDocumentBW = new BackgroundWorker();
            InitializeComponent();
            _generateDocumentBW.DoWork += DoWork_GenerateDocument;
            _generateDocumentBW.RunWorkerCompleted += RunWorkerCompleted_GenerateDocument;
        }

        /// <summary>
        /// This method serves action on storno button click in GenerateForm.
        /// </summary>
        /// <param name="sender">Sender.</param>
        /// <param name="e">Event arguments.</param>
        private void StornoGenerateButton_Click(object sender, EventArgs e)
        {
            Close();
        }

        /// <summary>
        /// This method serves action on choose data file button click in GenerateForm.
        /// It provides to load input data file.
        /// </summary>
        /// <param name="sender">Sender.</param>
        /// <param name="e">Event arguments.</param>
        private void ChooseDataFileButton_Click(object sender, EventArgs e)
        {
            _dataFilePath = _dialogs.DataFileChooseDialog();
            string fileName = Path.GetFileName(_dataFilePath);

            chooseDataFileLabel.Text = fileName;
            _isDataFilePathChosen = true;

            if (IsPathChosen)
            {
                generateButton.Enabled = true;
            }
        }

        /// <summary>
        /// This method serves action on choose new document file button click in GenerateForm.
        /// It provides to load new document file.
        /// </summary>
        /// <param name="sender">Sender.</param>
        /// <param name="e">Event arguments.</param>
        private void ChooseOutputButton_Click(object sender, EventArgs e)
        {
            _generatedDocFilePath = _dialogs.GeneratedDocSaveDialog();
            string fileName = Path.GetFileName(_generatedDocFilePath);

            chooseOutputLabel.Text = fileName;
            _isGeneratedDocFilePathChosen = true;

            if (IsPathChosen)
            {
                generateButton.Enabled = true;
            }
        }

        /// <summary>
        /// This method serves action on generate button click in GenerateForm.
        /// </summary>
        /// <param name="sender">Sender.</param>
        /// <param name="e">Event arguments.</param>
        private void GenerateButton_Click(object sender, EventArgs e)
        {
            Enabled = false;

            if (IsPathChosen)
            {
                _waitingForm = new WaitingForm();
                _waitingForm.Show();

                _generateDocumentBW.RunWorkerAsync();
            }
            else
            {
                string errorMessage = "Path(s) are not specified. Specify the path(s) and try it again.";
                MessageBox.Show(errorMessage, "Error: Path(s) not specified", MessageBoxButtons.OK, MessageBoxIcon.Error);
                Enabled = true;
            }

        }

        /// <summary>
        /// This method provides to generate new Word document from document template and input data. 
        /// </summary>
        /// <param name="sender">Sender.</param>
        /// <param name="e">Event arguments.</param>
        private void DoWork_GenerateDocument(object sender, DoWorkEventArgs e)
        {
            DocumentManager documentManager = DocumentManager.Instance;
            Thread.CurrentThread.CurrentUICulture = new System.Globalization.CultureInfo("en-US");

            try
            {
                documentManager.LoadTempTemplate(_currentTemplate);

                documentManager.Preprocess();

                documentManager.CreateDocument(_generatedDocFilePath);

                documentManager.GenerateDocument(_dataFilePath);

                documentManager.Close();

                string infoMessage = "Document generation successfull.";
                MessageBox.Show(infoMessage, "Information: Document Generation", MessageBoxButtons.OK, MessageBoxIcon.Information);
            }
            catch (Exception ex)
            {
                documentManager.Close();

                string infoMessage = "Document generation failed. Generated document is possibly invalid.\nError:\n" + ex.Message;
                MessageBox.Show(infoMessage, "Error: Document Generation", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }

            try
            {
                documentManager.CloseWordProcesses();
            }
            catch (Exception ex)
            {
                string infoMessage = "Unable to close MS Word processes: " + ex.Message;
                MessageBox.Show(infoMessage, "Warning: Template Builder", MessageBoxButtons.OK, MessageBoxIcon.Warning);
            }
        }

        /// <summary>
        /// This method is started after DoWork_GenerateDocument method is completed.
        /// It cleanups after background worker finished.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void RunWorkerCompleted_GenerateDocument(object sender, RunWorkerCompletedEventArgs e)
        {
            _waitingForm.Close();
            Enabled = true;
            Close();
        }
    }
}
