﻿using DocsGen.Services;
using System;
using System.Collections.Generic;
using System.Drawing;
using System.Windows.Forms;
using TemplateBuilder.Model.Entities;
using TemplateBuilder.Model.Entities.Metadata;
using TemplateBuilder.Model.Entities.NodeObjects;
using TemplateBuilder.Model.Entities.NodeObjects.Directives;
using TemplateBuilder.Model.Services;

namespace TemplateBuilder.Forms
{
    /// <summary>
    /// This partial class serves actions for TableForeachForm. 
    /// The form mediates GUI for the initial table creating 
    /// for tableforeach directive.
    /// </summary>
    public partial class TableForeachForm : Form
    {
        /// <summary>
        /// Instance of NodeManager.
        /// </summary>
        private NodeManager NodeManager { get; set; }

        /// <summary>
        /// List of attributes.
        /// </summary>
        private List<NodeObjectAttribute> Attributes { get; set; }

        /// <summary>
        /// List of text boxes viewed in TableForeachForm.
        /// </summary>
        private List<TextBox> TextBoxes { get; set; }

        /// <summary>
        /// List of combo boxes viewed in TableForeachForm.
        /// </summary>
        private List<ComboBox> ComboBoxes { get; set; }

        /// <summary>
        /// Constructor of the TableForeachForm class.
        /// </summary>
        /// <param name="nodeManager">Instance of NodeManager.</param>
        public TableForeachForm(NodeManager nodeManager)
        {
            NodeManager = nodeManager;
            InitializeComponent();

            try
            {
                InitializeCustomComponent();
            }
            catch(Exception ex)
            {
                string infoMessage = "Opening Table Foreach Form failed.\nError:\n" + ex.Message;
                MessageBox.Show(infoMessage, "Error: Table Foreach Form", MessageBoxButtons.OK, MessageBoxIcon.Error);
                Close();
            }
            
        }

        /// <summary>
        /// This method initializes custom components of the TableForeachForm.
        /// </summary>
        private void InitializeCustomComponent()
        {
            MetadataObject dataObject = GetDataObjectForTableForeach();

            if (IsDataObjectCorrect(dataObject))
            {
                Attributes = GenerateNodeObjectAttributes(dataObject);
            }

            if (Attributes == null || Attributes.Count <= 0)
            {
                throw new AttributeException(ErrorType.ATTRIBUTES_EMPTY);
            }

            ComboBoxes = new List<ComboBox>();
            TextBoxes = new List<TextBox>();
            
            tableComboBox1.Items.AddRange(Attributes.ToArray());
            ComboBoxes.Add(tableComboBox1);
            tableComboBox1.SelectedIndex = 0;
            tableTextBox1.Text = Attributes[0].Label;
            TextBoxes.Add(tableTextBox1);

            tableComboBox2.Items.AddRange(Attributes.ToArray());
            ComboBoxes.Add(tableComboBox2);
            tableComboBox2.SelectedIndex = 1;
            tableTextBox2.Text = Attributes[1].Label;
            TextBoxes.Add(tableTextBox2);

            tableComboBox3.Items.AddRange(Attributes.ToArray());
            ComboBoxes.Add(tableComboBox3);
            tableComboBox3.SelectedIndex = 2;
            tableTextBox3.Text = Attributes[2].Label;
            TextBoxes.Add(tableTextBox3);
        }

        /// <summary>
        /// This method founds out attributes relates to object in metadata.
        /// </summary>
        /// <param name="dataObject">Metadata object.</param>
        /// <returns>List of found attributes.</returns>
        private List<NodeObjectAttribute> GenerateNodeObjectAttributes(MetadataObject dataObject)
        {
            NodeManager currentNodeManager;
            List<NodeObjectAttribute> attributes = new List<NodeObjectAttribute>();

            TreeNode treeRoot = new TreeNode(dataObject.Name)
            {
                Tag = new MetadataParams(dataObject)
            };

            foreach (MetadataAttribute currentAttributeMetadata in dataObject.Attributes)
            {
                TreeNode treeNode = new TreeNode
                {
                    Name = currentAttributeMetadata.Name,
                    Tag = new MetadataParams(
                        currentAttributeMetadata, 
                        ((NodeObjectTableForeachDirective)((NodeObjectElement)NodeManager.NodeObject).Directive).ControllingVariable, 
                        NodeManager.NodeObjectFullName)
                };

                treeRoot.Nodes.Add(treeNode);

                currentNodeManager = new NodeManager(treeNode);
                attributes.Add((NodeObjectAttribute)currentNodeManager.NodeObject);
            }

            return attributes;
        }

        /// <summary>
        /// This method resolves if input metadata object is correct.
        /// </summary>
        /// <param name="dataObject">Metadata object.</param>
        /// <returns>true - if metadata object is correct; false - otherwise.</returns>
        private bool IsDataObjectCorrect(MetadataObject dataObject)
        {
            if (dataObject.Descendants != null)
            {
                return false;
            }

            if (dataObject.Attributes == null || dataObject.Attributes.Count <= 0)
            {
                return false;
            }

            return true;
        }

        /// <summary>
        /// This method founds out metadata object for node relates to current NodeManager instance.
        /// </summary>
        /// <returns>Found metadata object.</returns>
        private MetadataObject GetDataObjectForTableForeach()
        {
            MetadataObject dataObject;

            MetadataObject root = Globals.ThisAddIn.MetadataLoader.Root;
            dataObject = MetadataExecutor.FindElementByPath(root, NodeManager.NodeObjectFullName.Split('.'));


            if (dataObject.Descendants == null || dataObject.Descendants.Count <= 0)
            {
                throw new MetadataException(ErrorType.DATA_OBJECT_NOT_FOUND);
            }

            return dataObject.Descendants[0];
        }

        /// <summary>
        /// This method serves action on add column button click in TableForeachForm.
        /// It adds new column to the table preview.
        /// </summary>
        /// <param name="sender">Sender.</param>
        /// <param name="e">Event arguments.</param>
        private void AddColumnButton_Click(object sender, EventArgs e)
        {
            int lastColumnIndex = tableLayoutPanel1.ColumnCount;

            tableLayoutPanel1.ColumnCount++;

            tableLayoutPanel1.ColumnStyles.Add(new ColumnStyle(SizeType.AutoSize));

            tableLayoutPanel1.Controls.Add(new Label() {
                Anchor = AnchorStyles.Top | AnchorStyles.Bottom | AnchorStyles.Left | AnchorStyles.Right,
                Size = new Size(121, 29),
                TextAlign = ContentAlignment.MiddleCenter,
                Text = (tableLayoutPanel1.ColumnCount - 1).ToString()
            }, lastColumnIndex, 0);

            TextBox newTextBox = new TextBox()
            {
                AllowDrop = true,
                Anchor = AnchorStyles.Top | AnchorStyles.Bottom | AnchorStyles.Left | AnchorStyles.Right,
                AcceptsReturn = true,
                Size = new Size(121, 20),
                Text = Attributes[(lastColumnIndex - 1) % Attributes.Count].Label
            };

            tableLayoutPanel1.Controls.Add(newTextBox, lastColumnIndex, 1);

            ComboBox newComboBox = new ComboBox()
            {
                Anchor = AnchorStyles.Top | AnchorStyles.Bottom | AnchorStyles.Left | AnchorStyles.Right,
                FormattingEnabled = true,
                Size = new Size(121, 21),
                DataSource = Attributes.ToArray()
            };

            tableLayoutPanel1.Controls.Add(newComboBox, lastColumnIndex, 2);
            newComboBox.SelectedIndex = (lastColumnIndex - 1) % Attributes.Count;

            TextBoxes.Add(newTextBox);
            ComboBoxes.Add(newComboBox);

            removeColumnButton.Enabled = true;
        }

        /// <summary>
        /// This method serves action on remove column button click in TableForeachForm.
        /// It removes a column from the table preview.
        /// </summary>
        /// <param name="sender">Sender.</param>
        /// <param name="e">Event arguments.</param>
        private void RemoveColumnButton_Click(object sender, EventArgs e)
        {
            int lastColumntIndex = tableLayoutPanel1.ColumnCount - 1;

            if (lastColumntIndex == 0)
            {
                return;
            }

            for (int i = 0; i < tableLayoutPanel1.RowCount; i++)
            {
                Control currentControl = tableLayoutPanel1.GetControlFromPosition(lastColumntIndex, i);
                tableLayoutPanel1.Controls.Remove(currentControl);
            }

            tableLayoutPanel1.ColumnCount--;

            TextBoxes.RemoveAt(TextBoxes.Count - 1);
            ComboBoxes.RemoveAt(ComboBoxes.Count - 1);

            if (tableLayoutPanel1.ColumnCount == 1)
            {
                removeColumnButton.Enabled = false;
            }
        }

        /// <summary>
        /// This method serves action on storno button click in TableForeachForm.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void StornoButton_Click(object sender, EventArgs e)
        {
            Close();
        }

        /// <summary>
        /// This method serves action on add button click in TableForeachForm.
        /// It generates initial table into the document template.
        /// </summary>
        /// <param name="sender">Sender.</param>
        /// <param name="e">Event arguments.</param>
        private void AddButton_Click(object sender, EventArgs e)
        {
            try
            {
                string[] headerRowValues = GetHeaderRowValues();
                string[] valueRowValues = GetValueRowValues();

                WordManager.InsertMark(NodeManager.TemplateMark);
                WordManager.InsertForeachTableWithValues(headerRowValues, valueRowValues);
                WordManager.InsertMark("#end");
            }
            catch (Exception ex)
            {
                string infoMessage = "Inserting tableforeach directive failed.\nError:\n" + ex.Message;
                MessageBox.Show(infoMessage, "Error: Table Foreach Form", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }

            Close();
        }

        /// <summary>
        /// This method gets values of 'value row' from table preview.
        /// </summary>
        /// <returns>Array of values.</returns>
        private string[] GetValueRowValues()
        {
            List<string> valueRowValues = new List<string>();

            foreach (ComboBox currentComboBox in ComboBoxes)
            {
                valueRowValues.Add(((NodeObjectAttribute)currentComboBox.SelectedItem).TemplateMark);
            }

            return valueRowValues.ToArray();
        }

        /// <summary>
        /// This method gets values of 'header row' from table preview.
        /// </summary>
        /// <returns>Array of values.</returns>
        private string[] GetHeaderRowValues()
        {
            List<string> headerRowValues = new List<string>();

            foreach (TextBox currentTextBox in TextBoxes)
            {
                headerRowValues.Add(currentTextBox.Text);
            }

            return headerRowValues.ToArray();
        }
    }
}
