﻿using System.Windows.Forms;
using TemplateBuilder.Model.Entities;
using TemplateBuilder.Model.Entities.Metadata;

namespace TemplateBuilder.Model.Services
{
    /// <summary>
    /// This static class contains methods for work with Metadata Tree View.
    /// </summary>
    public static class MetadataExecutor
    {
        /// <summary>
        /// This method provides to process entered node of Metadata Tree View.
        /// </summary>
        /// <param name="parent">Parent node of Metadata Tree View.</param>
        /// <param name="node">Node of MetadataObject.</param>
        public static void ExecuteNode(TreeNode parent, MetadataObject node)
        {
            ExecuteNode(parent, node, null, null);
        }

        /// <summary>
        /// This method recursively processes entered node of Metadata Tree View.
        /// It resolves its attributes and elements.
        /// </summary>
        /// <param name="parent">Parent node of Metadata Tree View.</param>
        /// <param name="node">Node of MetadataObject.</param>
        /// <param name="controlVariableId">Identifier of the controlling variable.</param>
        /// <param name="controlArray">Identifier of the controlling array.</param>
        public static void ExecuteNode(TreeNode parent, MetadataObject node, 
            string controlVariableId, string controlArray)
        {

            if (node.IsArray)
            {
                return;
            }

            if (!(node.Attributes == null || node.Attributes.Count == 0))
            {
                foreach (var attribute in node.Attributes)
                {
                    if (!IsUnique(attribute.Name, parent))
                    {
                        throw new MetadataException(ErrorType.METADATA_UNIQUE_NAME_ATTR);
                    }

                    TreeNode newNode;

                    if (attribute.Example != null)
                    {
                        newNode = parent.Nodes.Add(attribute.Name, attribute.Label + " (example: " + attribute.Example + ")");
                    }
                    else
                    {
                        newNode = parent.Nodes.Add(attribute.Name, attribute.Label);
                    }

                    newNode.Name = attribute.Name;
                    newNode.Tag = new MetadataParams(attribute, controlVariableId, controlArray);

                    SetIcon(newNode);
                }
            }

            if (!(node.Descendants == null || node.Descendants.Count == 0))
            {
                foreach (var element in node.Descendants)
                {
                    if (!IsUnique(element.Name, parent))
                    {
                        throw new MetadataException(ErrorType.METADATA_UNIQUE_NAME_OBJ);
                    }

                    TreeNode newParent = parent.Nodes.Add(element.Name, element.Label);
                    newParent.Name = element.Name;
                    newParent.Tag = new MetadataParams(element, controlVariableId, controlArray);
                    SetIcon(newParent);

                    ExecuteNode(newParent, element, controlVariableId, controlArray);
                }
            }

            if (node.IsExpanded)
            {
                parent.Expand();
            }

        }

        /// <summary>
        /// This method sets icon of the node in Metadata Tree View.
        /// </summary>
        /// <param name="node">Node to set icon.</param>
        private static void SetIcon(TreeNode node)
        {
            object nodeTag = ((MetadataParams)node.Tag).MetadataClass;

            if (nodeTag is MetadataAttribute)
            {
                switch(((MetadataAttribute)nodeTag).DataType)
                {
                    case EDataType.DateTime:
                        node.ImageKey = Constants.DATE_TIME_IMAGE_KEY;
                        node.SelectedImageKey = Constants.DATE_TIME_IMAGE_KEY;
                        break;
                    case EDataType.Integer:
                        node.ImageKey = Constants.NUMBER_IMAGE_KEY;
                        node.SelectedImageKey = Constants.NUMBER_IMAGE_KEY;
                        break;
                    case EDataType.String:
                        node.ImageKey = Constants.TEXT_IMAGE_KEY;
                        node.SelectedImageKey = Constants.TEXT_IMAGE_KEY;
                        break;
                    default:
                        node.ImageKey = Constants.DEFAULT_IMAGE_KEY;
                        node.SelectedImageKey = Constants.DEFAULT_IMAGE_KEY;
                        break;
                }
            }
            else
            {
                if (((MetadataObject)nodeTag).IsArray) 
                {
                    node.ImageKey = Constants.DIRECTIVE_IMAGE_KEY;
                    node.SelectedImageKey = Constants.DIRECTIVE_IMAGE_KEY;
                }
                else
                {
                    node.ImageKey = Constants.NODE_IMAGE_KEY;
                    node.SelectedImageKey = Constants.NODE_IMAGE_KEY;
                }
                
            }
        }

        /// <summary>
        /// This method resolves if node is unique within the level of nodes hierarchy.
        /// </summary>
        /// <param name="key">Key of the node.</param>
        /// <param name="node">Parent node.</param>
        /// <returns>true - if node is unique; false - otherwise.</returns>
        private static bool IsUnique(string key, TreeNode node)
        {
            if (node.Nodes.ContainsKey(key))
            {
                return false;
            }

            return true;
        }

        /// <summary>
        /// This method finds node in metadata by controlling array.
        /// </summary>
        /// <param name="node">Parent node in metadata.</param>
        /// <param name="controlArray">Controlling array.</param>
        /// <returns>Found metadata node.</returns>
        public static MetadataObject FindElementByPath(MetadataObject node, string[] controlArray)
        {
            MetadataObject elementNode = node;
            bool found = false;

            if (!elementNode.Name.Equals(controlArray[0]))
            {
                return null;
            }

            for (int i = 1; i < controlArray.Length; i++)
            {
                found = false;

                if (elementNode.Descendants == null || elementNode.Descendants.Count == 0)
                    return null;

                foreach (MetadataObject element in elementNode.Descendants)
                {
                    if (element.Name.Equals(controlArray[i]))
                    {
                        found = true;
                        elementNode = element;
                    }
                }

                if (!found)
                {
                    return null;
                }
            }

            return elementNode;
        }
    }
}
